VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsFileComm"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private mStrOutFile As String   ' Output channel file
Private mStrInFile As String    ' Input channel file
Private mBlnCancel As Boolean   ' True if to cancel an operation

Public Property Let OutFile(strFile As String)
    ' Set the output file name.
    mStrOutFile = strFile
End Property

Public Property Get OutFile() As String
    ' Return the output file.
    OutFile = mStrOutFile
End Property

Public Property Let InFile(strFile As String)
    ' Set the input file name.
    mStrInFile = strFile
End Property

Public Property Get InFile() As String
    ' Return the input file.
    InFile = mStrInFile
End Property

Public Property Let Cancel(blnCancel As Boolean)
    ' Set the local variable indicating that the current operation be cancelled.
    mBlnCancel = blnCancel
End Property

Public Sub SendData(strData As String)
    ' Send data to the output file.
    Dim intFileNum As Integer
    
    On Error GoTo SendDataErrorHandler
    
    ' If no output file specified then exit.
    If Trim$(mStrOutFile) = "" Then Exit Sub
    
    ' Attempt to open the file locked
    intFileNum = FreeFile
    
    ' If the data has been read (archive bit turned off),
    ' open the file for Output in order to overwrite existing contents.
    ' Otherwise, append data to the file.
    mBlnCancel = False
TryToOpenAgain:
    
    If (GetAttr(mStrOutFile) And vbArchive) = 0 Then
        Open mStrOutFile For Output Lock Read Write As #intFileNum
    Else
        Open mStrOutFile For Append Lock Read Write As #intFileNum
    End If
    
    Print #intFileNum, strData & vbCr;
    Close #intFileNum
    
    ' The archive bit is automatically set, indicating the file contains new data.
    'SetAttr mStrOutFile, vbArchive
    
    Exit Sub
'________________________
SendDataErrorHandler:
    
    Select Case Err.Number
    Case 53     ' File not found.  Create it and resume.
        Open mStrOutFile For Output As #intFileNum
        Close intFileNum
        Resume
    Case 70     ' If the file is locked, release the processor and try again.
        DoEvents
        ' Check if cancelled
        If mBlnCancel = True Then
            mBlnCancel = False
            Exit Sub
        End If
        Resume TryToOpenAgain
    Case Else
        MsgBox "Error (SendData) " & CStr(Err.Number) & ": " & Err.Description
    End Select
    
End Sub

Public Sub GetData(strData As String)
    ' Get data from the input file.
    Dim intFileNum As Integer, lngFileLen As Long
    
    On Error GoTo GetDataErrorHandler
            
    ' If no input file specified then exit.
    If Trim$(mStrInFile) = "" Then Exit Sub
    
    ' Exit if archive bit is not set, implying file contains no new data.
    If (GetAttr(mStrInFile) And vbArchive) = 0 Then Exit Sub

' Another way to know that the file has changed is to look at its date and time.
'    Dim datTmp As Date
'    Static datFile As Date
'    datTmp = FileDateTime(mStrInFile)
'    If datTmp <= datFile Then Exit Sub
'    datFile = datTmp        ' Update the latest file date-time stamp

    ' Attempt to open the file locked and read all data.
    strData = ""            ' Clear data variable
    intFileNum = FreeFile
    Open mStrInFile For Input Lock Read Write As #intFileNum
    lngFileLen = LOF(intFileNum)
    If lngFileLen > 0& Then strData = Input(lngFileLen, #intFileNum)
    Close #intFileNum
    
    ' Clear archive bit.
    SetAttr mStrInFile, vbNormal
    
    Exit Sub
'________________________
GetDataErrorHandler:
    
    Select Case Err.Number
    Case 53     ' If File Not Found, then exit.
    Case 70     ' If the file is locked, release the processor and try again.
        DoEvents
        ' Check if cancelled
        If mBlnCancel = True Then
            mBlnCancel = False
            Exit Sub
        End If
        Resume
    Case Else
        MsgBox "Error (GetData) " & CStr(Err.Number) & ": " & Err.Description
    End Select

End Sub

