VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Knob"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private WithEvents mPct As PictureBox   ' The PictureBox on which to draw the knob
Attribute mPct.VB_VarHelpID = -1
Private mBlnSelecting As Boolean        ' Interactive state variable
Private mSngValue As Single             ' Value of the knob [0, 100]
Const Pi As Single = 3.14159            ' The constant Pi

' Declare the change event which is raised when the knob value changes.
Public Event Change(ByVal sngValue As Single)

Property Set PictureBox(pctBox As PictureBox)
    Dim sngWidth As Single, sngHeight As Single, sngAspect As Single

    ' Set the PictureBox on which the gauge should be drawn
    Set mPct = pctBox
    
    ' Get PictureBox dimensions
    sngWidth = mPct.Width
    sngHeight = mPct.Height
    sngAspect = sngWidth / sngHeight
    
    ' Set the coordinates of the PictureBox.
    If sngAspect > 1# Then
        mPct.Scale (-1.4 * sngAspect, 1.4)-(1.4 * sngAspect, -1.4)
    Else
        mPct.Scale (-1.4, 1.4 * sngAspect)-(1.4, -1.4 * sngAspect)
    End If
    
    ' Save the background image and initially draw the knob.
    SetBackground
    DrawKnob
    
End Property

Public Sub DrawKnob()
    Dim sng As Single, sngAngle As Single
    Dim sngX1 As Single, sngY1 As Single, sngX2 As Single, sngY2 As Single
    
    ' Reset the background
    mPct.Cls
    
    ' Draw the knob mark
    mPct.DrawWidth = 2
    sngAngle = (225# - (mSngValue * 2.7)) * (Pi / 180#)
    sngX1 = 0.6 * Cos(sngAngle)
    sngY1 = 0.6 * Sin(sngAngle)
    sngX2 = 1# * Cos(sngAngle)
    sngY2 = 1# * Sin(sngAngle)
    mPct.Line (sngX1, sngY1)-(sngX2, sngY2), vbBlack
    
End Sub

Private Sub SetBackground()
    ' Set the knob background
    Dim sng As Single, sngAngle As Single
    Dim sngX1 As Single, sngY1 As Single, sngX2 As Single, sngY2 As Single
    
    mPct.AutoRedraw = True          ' Begin to maintain a persistent graphic
    mPct.BackColor = vbButtonFace   ' Erase all and set the background color
    mPct.FillStyle = 0 ' solid
    mPct.DrawWidth = 2
    
    ' Draw the circle
    mPct.FillColor = vbWhite
    mPct.Circle (0#, 0#), 1, vbGrayText
    
    ' Draw the tick marks
    mPct.DrawWidth = 4
    For sng = -45# To 225# Step 45#
        sngAngle = sng * (Pi / 180#)
        sngX1 = 1.1 * Cos(sngAngle)
        sngY1 = 1.1 * Sin(sngAngle)
        sngX2 = 1.2 * Cos(sngAngle)
        sngY2 = 1.2 * Sin(sngAngle)
        mPct.Line (sngX1, sngY1)-(sngX2, sngY2), vbRed
    Next sng
    
    ' Label the first and last tick
    mPct.Font.Size = 16
    mPct.CurrentX = -1#
    mPct.CurrentY = -1#
    mPct.Print "0"
    
    mPct.CurrentX = 0.7
    mPct.CurrentY = -1#
    mPct.Print "100"
    
    ' Save the generated background image
    mPct.Picture = mPct.Image   ' Copy the persistent graphic to the Picture property
    mPct.AutoRedraw = False     ' Turn off the persistent graphic for better performance

End Sub

Public Property Get Value() As Single
    ' Return the knob Value property
    Value = mSngValue
End Property

Public Property Let Value(sngValue As Single)
    ' Set the value to be displayed by the knob.
    
    ' Clip to the range [0, 100]
    If sngValue < 0# Then
        mSngValue = 0#
    ElseIf sngValue > 100# Then
        mSngValue = 100#
    Else
        mSngValue = sngValue
    End If
    
    ' Redraw the knob
    DrawKnob
    
    ' Raise an event indicating that the value has changed
    RaiseEvent Change(mSngValue)
    
End Property

Private Sub mPct_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' Change the knob setting to appropriate value
    mBlnSelecting = True
    Value = CPositionToValue(X, Y)
End Sub

Private Sub mPct_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' Change the knob setting to appropriate value
    If mBlnSelecting = False Then Exit Sub    ' If not setting then exit
    Value = CPositionToValue(X, Y)
End Sub

Private Sub mPct_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' Exit setting state
    mBlnSelecting = False
End Sub

Private Function CPositionToValue(X As Single, Y As Single) As Single
    ' Convert the X-Y mouse position to a knob value
    Dim sngValue As Single
    
    If X > 0# Then
        sngValue = Atn(Y / X)
    ElseIf X < 0# Then
        sngValue = Atn(Y / X) + 3.1416
    End If
    
    CPositionToValue = 100# - ((sngValue * (180# / 3.1416)) + 45#) / 2.7
    
End Function

Private Sub mPct_Paint()
    ' Regenerate the knob image
    DrawKnob
End Sub
