VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Thermometer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private WithEvents mPct As PictureBox       ' Reference to PictureBox object
Attribute mPct.VB_VarHelpID = -1
Private mBlnSelecting As Boolean            ' Selection state

Private mSngTemp As Single                  ' Internal store of temperature value
Private mSngThreshold As Single             ' Internal store of threshold value

' Change event raised each time temperature changes.
Public Event Change(ByVal sngTemperature As Single)

' OverThreshold event raised when the threshold is crossed in an upward direction.
Public Event OverThreshold(ByVal sngTemperature As Single)

' UnderThreshold event raised when the threshold is crossed in a downward direction.
Public Event UnderThreshold(ByVal sngTemperature As Single)

Public Property Set PictureBox(pctBox As PictureBox)
    ' Set up the PictureBox on which to draw the Thermometer
    Dim sngHWidth As Double, I As Integer
    
    Set mPct = pctBox            ' Set the internal picture box on which to draw
                                 ' Set the coordinate system
    sngHWidth = (mPct.Width / mPct.Height) * 70#
    mPct.Scale (-sngHWidth, 110#)-(sngHWidth, -30#)
    
    ' Create and save the background.  Initially draw the thermometer.
    SetBackground
    DrawThermometer
    
End Property

Private Sub SetBackground()
    ' Draw and save the background
    Dim I As Integer
    
    mPct.AutoRedraw = True          ' Initiate persistent graphic
    mPct.BackColor = vbWhite
    
    ' Draw background
    mPct.FillStyle = vbSolid
    mPct.FillColor = vbRed                  ' Draw Bulb
    mPct.Circle (0, -8), 10, vbBlack
    
    mPct.FillColor = vbWhite                ' Draw Stem
    mPct.Line (-5, 0)-(5, 100), vbWhite, B  ' Fill Stem
    mPct.Line (-5, 0)-(-5, 100), vbBlack    ' Draw sides
    mPct.Line (5, 0)-(5, 100), vbBlack
    
    mPct.FillColor = vbBlue                 ' Draw Threshold region
    mPct.Line (-5, mSngThreshold)-(5, 100), vbBlack, B
    
    For I = 0 To 100 Step 20                ' Draw Temperature Scale
        mPct.Line (5, I)-(8, I), vbBlack
        mPct.Print I
    Next I
    
    ' Save background
    mPct.Picture = mPct.Image
    mPct.AutoRedraw = False

End Sub

Public Property Get Temperature() As Single
    Temperature = mSngTemp       ' Return internal temperature value
End Property

Public Property Let Temperature(sngNewTemp As Single)
    ' Set the Thermometer's temperature
    Dim sngOldTemp As Single
                                    ' Clip temperature to reasonable values
    If sngNewTemp < 0# Then sngNewTemp = 0#
    If sngNewTemp > 100# Then sngNewTemp = 100#
    
    sngOldTemp = mSngTemp           ' Save old temperature
    mSngTemp = sngNewTemp           ' Set new temperature
    
    DrawThermometer                 ' Update the drawing
    
    RaiseEvent Change(mSngTemp)     ' Raise the Change event
    If sngOldTemp <= mSngThreshold And sngNewTemp > mSngThreshold Then RaiseEvent OverThreshold(mSngTemp)
    If sngOldTemp >= mSngThreshold And sngNewTemp < mSngThreshold Then RaiseEvent UnderThreshold(mSngTemp)
    
End Property

Public Property Get Threshold() As Single
    ' Return the thermometer threshold
    Threshold = mSngThreshold
End Property

Public Property Let Threshold(sngNewThresh As Single)
    ' Set the Threshold property value
    
    ' Clip threshold to reasonable values
    If sngNewThresh < 0# Then sngNewThresh = 0#
    If sngNewThresh > 100# Then sngNewThresh = 100#
    
    ' Set the new value
    mSngThreshold = sngNewThresh
    
    ' Reset the new background and redraw the Thermometer
    SetBackground
    DrawThermometer

End Property

Private Sub DrawThermometer()
    ' Draw the thermometer on the PictureBox
    
    ' Restore the background
    mPct.Cls
    
    ' Redraw the stem of the thermometer.
    mPct.FillColor = vbRed                      ' Fill with mercury
    mPct.Line (-5, 0)-(5, mSngTemp), vbRed, B
    mPct.Line (-5, 0)-(-5, mSngTemp), vbBlack   ' Draw sides
    mPct.Line (5, 0)-(5, mSngTemp), vbBlack

End Sub

Private Sub Class_Initialize()
    ' Initialize object properties
    mSngTemp = 0#
    mSngThreshold = 70#
End Sub

Private Sub mPct_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    mBlnSelecting = True            ' Enter selecting state
    Temperature = Y                 ' Set temperature
End Sub

Private Sub mPct_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If mBlnSelecting = False Then Exit Sub
    Temperature = Y                 ' Update temperature
End Sub

Private Sub mPct_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    mBlnSelecting = False           ' Exit selecting state
End Sub

